package de.intevation.flys.artifacts.context;

import org.apache.log4j.Logger;

import org.hibernate.Session;
import org.hibernate.SessionFactory;

import de.intevation.flys.backend.SessionFactoryProvider;

import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.CallContext.Listener;


/**
 * This CallContextListener is used to initialize a ThreadLocal variable in
 * each CallContext (for each request) that holds Sessions.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class SessionCallContextListener implements Listener {

    public static final String SESSION_HOLDER_KEY = "context.session.holder";

    /** The logger that is used in this class.*/
    private static Logger logger =
        Logger.getLogger(SessionCallContextListener.class);


    public SessionCallContextListener() {
    }


    /**
     * Initializes a ThreadLocal variable that is used to hold sessions.
     *
     * @param context The CallContext.
     */
    public void init(CallContext context) {
        logger.debug("SessionCallContextListener.init");

        ThreadLocal<Session> sessionHolder = new ThreadLocal<Session>();

        SessionFactory sessionFactory =
            SessionFactoryProvider.createSessionFactory();

        Session session = sessionFactory.openSession();

        sessionHolder.set(session);

        context.putContextValue(SESSION_HOLDER_KEY, sessionHolder);
    }


    /**
     * Closes open sessions of the ThreadLocal variable opened in init().
     *
     * @param context The CallContext.
     */
    public void close(CallContext context) {
        logger.debug("SessionCallContextListener.close");

        ThreadLocal<Session> sessionHolder = (ThreadLocal<Session>)
            context.getContextValue(SESSION_HOLDER_KEY);

        Session session = sessionHolder.get();

        session.close();
        sessionHolder.remove();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
